function createInstanceFeatureForm(options) {
	var newInstanceFeatureForm = $('<form></form>');
	newInstanceFeatureForm.attr('id', 'newFeature');
	
	var featureNameLabel = $('<label></label>');
	featureNameLabel.attr('for', 'newFeature_featureName');
	featureNameLabel.html('Feature name');
	
	var featureNameField = $('<input></input>');
	featureNameField.attr('type', 'text');
	featureNameField.attr('size', 50);
	featureNameField.attr('id', 'newFeature_featureName');
	featureNameField.attr('name', 'newFeature_featureName');
	
	var featureNameContainer = $('<div></div>');
	featureNameContainer.addClass('formElement');
	featureNameContainer.append(featureNameLabel);
	featureNameContainer.append(featureNameField);
	
	newInstanceFeatureForm.append(featureNameContainer);
	
	var tagSetsHeaderText = $('<span></span>').html('Tag sets');
	var tagSetsContent = $('<div></div>');
	var tagSetsContainer = $('<div></div>');

	createCollapsibleMenu({
		'headerContent' : tagSetsHeaderText,
		'contentContainer' : tagSetsContent,
		'menuContainer' : tagSetsContainer,
		'initialState' : 'expanded'
	});
	
	createMultipleTagSetSelector({
		'target' : tagSetsContent,
		'idPrefix' : 'newFeature_',
		'description' : 'Please specify one or more sets of tags, such that one set must be present on an instance in order to use this feature. Each tag set should be a comma-separated list of tags, and you may use the drop-down selector to insert tags that are already defined in HAL.'
	});
	
	newInstanceFeatureForm.append(tagSetsContainer);
	
	var domainHeaderText = $('<span></span>').html('Domain');
	var domainContent = $('<div></div>');
	var domainContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : domainHeaderText,
		'contentContainer' : domainContent,
		'menuContainer' : domainContainer,
		'initialState' : 'expanded'
	});
	
	createParameterDomainSpecification({
		'target' : domainContent,
		'parameter' : undefined,
		'idPrefix' : 'newFeature_domain_',
		'showDefaultValueField' : false,
		'showTransformationField' : true
	});
	
	newInstanceFeatureForm.append(domainContainer);
	
	var newInstanceFeatureStatusMessages = $('<div></div>');
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.click(function() {
		submitButton.attr('disabled', true);
		
		newInstanceFeatureStatusMessages.removeClass('error').removeClass('alert').addClass('success');
		var spinner = $('<div></div>').addClass('center').addClass('spinner');
		spinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').addClass('center'));
		newInstanceFeatureStatusMessages.html(spinner);
		
		var formData = newInstanceFeatureForm.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/getFormSnippet/instanceFeature/createInstanceFeature',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					newInstanceFeatureStatusMessages.removeClass('error').removeClass('alert').addClass('success');
					newInstanceFeatureStatusMessages.html('Instance feature defined successfully.');
					newInstanceFeatureStatusMessages.show();
					
					setTimeout("window.location.href = '/hal/'", 1000);
				} else {
					newInstanceFeatureStatusMessages.removeClass('success').removeClass('alert').addClass('error');
					newInstanceFeatureStatusMessages.html(response.errorMessage);
					newInstanceFeatureStatusMessages.show();
				}
			},
			'json'
		);
	});
	
	var cancelButton = $('<input></input>');
	cancelButton.attr('type', 'button');
	cancelButton.attr('value', 'Cancel');
	cancelButton.click(cancelForm);
	
	var newInstanceFeatureButtons = $('<div></div>');
	newInstanceFeatureButtons.addClass('center');
	newInstanceFeatureButtons.append(submitButton);
	newInstanceFeatureButtons.append(cancelButton);
	
	options.target.append(newInstanceFeatureForm);
	options.target.append(newInstanceFeatureStatusMessages);
	options.target.append('<hr>');
	options.target.append(newInstanceFeatureButtons);
}

function createAddFeaturesToInstanceSetForm(options) {
	var addFeaturesForm = $('<form></form>');
	addFeaturesForm.attr('id', 'addFeaturesToInstanceSet');
	
	var instanceSetSelectionLabel = $('<label></label>');
	instanceSetSelectionLabel.attr('for', 'addFeature_instanceSetName');
	instanceSetSelectionLabel.html('Instance set');
	
	var instanceSetSelectionDescription = $('<div></div>');
	instanceSetSelectionDescription.addClass('formFieldDescription');
	instanceSetSelectionDescription.html('');
	
	var instanceSetSelectionField = $('<select></select>');
	instanceSetSelectionField.attr('id', 'addFeature_instanceSetName');
	instanceSetSelectionField.attr('name', 'addFeature_instanceSetName');
	
	for (var i in options.instanceSets) {
		var opt = $('<option></option>');
		opt.attr('value', options.instanceSets[i]);
		opt.html(options.instanceSets[i]);
		
		instanceSetSelectionField.append(opt);
	}
	
	var instanceSetSelectionContainer = $('<div></div>');
	instanceSetSelectionContainer.addClass('formElement');
	instanceSetSelectionContainer.append(instanceSetSelectionLabel);
	instanceSetSelectionContainer.append(instanceSetSelectionDescription);
	instanceSetSelectionContainer.append(instanceSetSelectionField);
	
	addFeaturesForm.append(instanceSetSelectionContainer);
	
	var featuresHeaderText = $('<span></span>').html('Features');
	var featuresContent = $('<div></div>');
	var featuresContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : featuresHeaderText,
		'contentContainer' : featuresContent,
		'menuContainer' : featuresContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(featuresContainer);
	
	var instancesHeaderText = $('<span></span>').html('Select specific instances');
	var instancesContent = $('<div></div>');
	var instancesContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : instancesHeaderText,
		'contentContainer' : instancesContent,
		'menuContainer' : instancesContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(instancesContainer);
	
	addFeaturesForm.append(instancesContainer);
	addFeaturesForm.append(featuresContainer);
	
	instanceSetSelectionField.change(function() {	
		instancesContent.html('');
		
		featuresContent.html('');
		featuresContent.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
		
		// Need to get all of the instance names or hashes to display in a collapsed list with checkboxes for inclusion,
		// features will get added only to those instances.
		$.post('/hal/getFormSnippet/utilities/getInstances',
			{
				'hal_httpResponseContentType' : 'json',
				'instanceSet' : instanceSetSelectionField.val()
			},
			function(response) {
				if (response.successful) {
					for (var i in response.result.instances) {
						var elementLabel = $('<label></label>');
						elementLabel.attr('for', 'addFeature_instance_'+response.result.instances[i].hash);
						elementLabel.html(response.result.instances[i].name);
						
						var elementCheckbox = $('<input></input>');
						elementCheckbox.attr('type', 'checkbox');
						elementCheckbox.attr('checked', 'on');
						elementCheckbox.attr('id', 'addFeature_instance_'+response.result.instances[i].hash);
						elementCheckbox.attr('name', 'addFeature_instance_'+response.result.instances[i].hash);
						
						var element = $('<div></div>');
						element.addClass('formElementInline');
						element.append(elementLabel);
						element.append(elementCheckbox);
						
						instancesContent.append(element);
					}
					
					enableMenu(instancesContainer);
					
					enableMenu(featuresContainer);
					expandMenu(featuresContainer);
					
					$.post('/hal/getFormSnippet/utilities/getCompatibleFeatures',
							{
								'hal_httpResponseContentType' : 'json',
								'instanceSet' : instanceSetSelectionField.val()
							},
							function(response) {
								if (response.successful) {
									featuresContent.html('');
									
									createMultipleFeatureSelector({
										'target' : featuresContent,
										'idPrefix' : 'addFeature_',
										'features' : response.result.compatibleFeatures
									});
								}
							},
							'json'
						);
				}
			},
			'json'
		);
			

	});
	
	instanceSetSelectionField.change();
	
	var addFeaturesStatusMessages = $('<div></div>');
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Add features');
	submitButton.click(function() {
		submitButton.attr('disabled', true);

		addFeaturesStatusMessages.removeClass('error').removeClass('alert').addClass('success');
		var spinner = $('<div></div>').addClass('center').addClass('spinner');
		spinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').addClass('center'));
		addFeaturesStatusMessages.html(spinner);
		
		var formData = addFeaturesForm.serialize();
		formData = formData+'&hal_httpContentResponse=json';
		
		$.post('/hal/getFormSnippet/instanceFeature/addFeaturesToInstanceSet',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					addFeaturesStatusMessages.removeClass('error').removeClass('alert').addClass('success');
					addFeaturesStatusMessages.html('All features were added successfully.');
					addFeaturesStatusMessages.show();
					
					setTimeout("window.location.href = '/hal/';", 1000);
				} else {
					addFeaturesStatusMessages.removeClass('success').removeClass('alert').addClass('error');
					addFeaturesStatusMessages.html(response.errorMessage);
					addFeaturesStatusMessages.show();
				}
			},
			'json'
		);
	});
	
	var cancelButton = $('<input></input>');
	cancelButton.attr('type', 'button');
	cancelButton.attr('value', 'Cancel');
	cancelButton.click(cancelForm);
	
	var addFeaturesButtons = $('<div></div>');
	addFeaturesButtons.addClass('center');
	addFeaturesButtons.append(submitButton);
	addFeaturesButtons.append(cancelButton);
	
	options.target.append(addFeaturesForm);
	options.target.append(addFeaturesStatusMessages);
	options.target.append('<hr>');
	options.target.append(addFeaturesButtons);
}

function createFeatureExtractorSelector(options) {
	options.target.addClass('formElement');

	var extractorLabel = $('<label></label>');
	extractorLabel.attr('for', options.idPrefix+'extractorName');
	extractorLabel.html('Feature extractor');
	
	var extractorField = $('<select></select>');
	extractorField.attr('id', options.idPrefix+'extractorName');
	extractorField.attr('name', options.idPrefix+'extractorName');

	for (var i in options.featureExtractors) {
		var opt = $('<option></option>');
		opt.attr('value', options.featureExtractors[i].name+'_xXxversionxXx_'+options.featureExtractors[i].version);
		opt.html(options.featureExtractors[i].name+' ('+options.featureExtractors[i].version+')');
		
		extractorField.append(opt);
	}
	
	var removeExtractorButton = $('<input></input>');
	removeExtractorButton.attr('type', 'button');
	removeExtractorButton.attr('value', 'Remove');
	removeExtractorButton.click(function() {
		$(this).parent().remove();
	});
	
	options.target.append(extractorLabel);
	options.target.append(extractorField);
	options.target.append(removeExtractorButton);
}

function createMultipleFeatureExtractorSelector(options) {
	options.target.html('');
	var compatibleFeatureList = [];
	
	var outerContainer = $('<div></div>');
	outerContainer.hide();
	
	var description = $('<div></div>');
	description.addClass('formFieldDescription');
	description.html(options.description);
	outerContainer.append(description);
	
	var numExtractors = 0;
	var numExtractorsField = $('<input></input>');
	numExtractorsField.attr('type', 'hidden');
	numExtractorsField.attr('name', options.idPrefix+'numberOfFeatureExtractors');
	numExtractorsField.attr('id', options.idPrefix+'numberOfFeatureExtractors');
	numExtractorsField.attr('value', numExtractors);
	outerContainer.append(numExtractorsField);
	
	var extractors = $('<div></div>');
	outerContainer.append(extractors);
	
	var addClick = function() {
		var newExtractorDiv = $('<div></div>');
		extractors.append(newExtractorDiv);
		
		createFeatureExtractorSelector({
			'target' : newExtractorDiv,
			'featureExtractors' : compatibleFeatureList,
			'idPrefix' : options.idPrefix+'featureExtractors_'+numExtractors+'_',
			'removable' : true
		});
		
		numExtractors = numExtractors+1;
		numExtractorsField.val(numExtractors);
	}
	
	var addButton = $('<input></input>');
	addButton.attr('type', 'button');
	addButton.attr('value', 'Add feature extractor');
	addButton.click(addClick);
	
	var next = function() {
		var selectedExtractors = {};
		var extractedFeatures = {};
		
		extractors.find('select').each(function() {
			selectedExtractors[$(this).val()] = true;
		});
		
		for (var i in compatibleFeatureList) {
			var idString = compatibleFeatureList[i].name+'_xXxversionxXx_'+compatibleFeatureList[i].version;
			
			if (selectedExtractors[idString] === true) {
				for (var j in compatibleFeatureList[i].features) {
					extractedFeatures[compatibleFeatureList[i].features[j]] = true;
				}
			}
		}
		
		options.onNext(extractedFeatures);
	}
	
	var nextButton = $('<input></input>');
	nextButton.attr('type', 'button');
	nextButton.attr('value', 'Next');
	nextButton.click(next);
	
	var buttons = $('<div></div>');
	buttons.append('<hr>');
	buttons.append(addButton);
	buttons.append(nextButton);
	
	outerContainer.append(buttons);
	
	$.post('/hal/getFormSnippet/utilities/getCompatibleFeatureExtractors',
		{
			'hal_httpResponseContentType' : 'json',
			'instanceSet' : options.instanceSet
		},
		function(response) {
			if (response.successful) {
				compatibleFeatureList = response.result.compatibleFeatureExtractors
				
				outerContainer.show();
				options.target.html(outerContainer);
				options.onFinished(compatibleFeatureList);
			}
		},
		'json'
	);
}