function createPortfolioBasedSelectorForm(options) {
	var newPortfolioBasedSelectorForm = $('<form></form>');
	newPortfolioBasedSelectorForm.attr('id', 'newPortfolioBasedSelectorForm');
	
	var experimentInfoSectionHeaderText = $('<span></span>').html('Experiment information');
	var experimentInfoSectionContent = $('<div></div>');
	var experimentInfoSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : experimentInfoSectionHeaderText,
		'contentContainer' : experimentInfoSectionContent,
		'menuContainer' : experimentInfoSectionContainer,
		'initialState' : 'expanded'
	});
	
	newPortfolioBasedSelectorForm.append(experimentInfoSectionContainer);
	
	experimentNameLabel = $('<label></label>');
	experimentNameLabel.attr('for', 'newPortfolioBasedSelector_experimentName');
	experimentNameLabel.html('Experiment name')
	
	experimentNameField = $('<input></input>');
	experimentNameField.attr('name', 'newPortfolioBasedSelector_experimentName');
	experimentNameField.attr('id', 'newPortfolioBasedSelector_experimentName');
	experimentNameField.attr('size', 75);
	experimentNameField.attr('type', 'text');
	
	var experimentNameContainer = $('<div></div>');
	experimentNameContainer.addClass('formElement');
	experimentNameContainer.append(experimentNameLabel);
	experimentNameContainer.append(experimentNameField);
	
	experimentInfoSectionContent.append(experimentNameContainer);
	
	designProcedureLabel = $('<label></label>');
	designProcedureLabel.attr('for', 'newPortfolioBasedSelector_designProcedure');
	designProcedureLabel.html('Design procedure');
	
	designProcedureField = $('<select></select>');
	designProcedureField.attr('name', 'newPortfolioBasedSelector_designProcedure');
	designProcedureField.attr('id', 'newPortfolioBasedSelector_designProcedure');
	
	for (var i in options.compatibleAlgNamesAndVersions) {
		var algName = options.compatibleAlgNamesAndVersions[i]['name'];
		var algVers = options.compatibleAlgNamesAndVersions[i]['version'];
		
		var opt = $('<option></option>');
		opt.attr('value', algName+'_xXxversionxXx_'+algVers);
		opt.html(algName+' ('+algVers+')');
		
		designProcedureField.append(opt);
	}
	
	var designProcedureContainer = $('<div></div>');
	designProcedureContainer.addClass('formElement');
	designProcedureContainer.append(designProcedureLabel);
	designProcedureContainer.append(designProcedureField);
	
	experimentInfoSectionContent.append(designProcedureContainer);

	instanceDistributionLabel = $('<label></label>');
	instanceDistributionLabel.attr('for', 'newPortfolioBasedSelector_instanceDistribution');
	instanceDistributionLabel.html('Instance Distribution');
	
	var instanceDistributionField = $('<select></select>');
	instanceDistributionField.attr('name', 'newPortfolioBasedSelector_instanceDistribution');
	instanceDistributionField.attr('id', 'newPortfolioBasedSelector_instanceDistribution');
	instanceDistributionField.append($('<option></option>').attr('value', '').html('Populating...'));
	
	var instanceDistributionContainer = $('<div></div>');
	instanceDistributionContainer.addClass('formElement');
	instanceDistributionContainer.append(instanceDistributionLabel);
	instanceDistributionContainer.append(instanceDistributionField);
	
	experimentInfoSectionContent.append(instanceDistributionContainer);
	
	var featuresSectionHeaderText = $('<span></span>').html('Feature selection');
	var featuresSectionContent = $('<div></div>');
	var featuresSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : featuresSectionHeaderText,
		'contentContainer' : featuresSectionContent,
		'menuContainer' : featuresSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(featuresSectionContainer);
	
	newPortfolioBasedSelectorForm.append(featuresSectionContainer);
	
	var featureExtractorSectionHeaderText = $('<span></span>').html('Feature extractor selection');
	var featureExtractorSectionContent = $('<div></div>');
	var featureExtractorSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : featureExtractorSectionHeaderText,
		'contentContainer' : featureExtractorSectionContent,
		'menuContainer' : featureExtractorSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(featureExtractorSectionContainer);
	featuresSectionContent.append(featureExtractorSectionContainer);
	
	var featureExtractorSelectorContainer = $('<div></div>');
	featureExtractorSelectorContainer.hide();
	
	var featureExtractorSpinner = $('<div></div>');
	featureExtractorSpinner.addClass('spinner');
	featureExtractorSpinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
	featureExtractorSpinner.hide();
	
	featureExtractorSectionContent.append(featureExtractorSpinner);
	featureExtractorSectionContent.append(featureExtractorSelectorContainer);
	
	var extractedFeaturesSectionHeaderText = $('<span></span>').html('Extracted features');
	var extractedFeaturesSectionContent = $('<div></div>');
	var extractedFeaturesSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : extractedFeaturesSectionHeaderText,
		'contentContainer' : extractedFeaturesSectionContent,
		'menuContainer' : extractedFeaturesSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(extractedFeaturesSectionContainer);
	
	featuresSectionContent.append(extractedFeaturesSectionContainer);

	var reenableExtractorSelectionButton = $('<input></input>');
	reenableExtractorSelectionButton.attr('type', 'button');
	reenableExtractorSelectionButton.attr('value', 'Back');
	reenableExtractorSelectionButton.click(function() {
		featuresSectionButtons.hide();
		
		disableMenu(extractedFeaturesSectionContainer);
		collapseMenu(extractedFeaturesSectionContainer);
		
		enableMenu(featureExtractorSectionContainer);
		expandMenu(featureExtractorSectionContainer);
	});
	
	var proceedToAlgSelectionButton = $('<input></input>');
	proceedToAlgSelectionButton.attr('type', 'button');
	proceedToAlgSelectionButton.attr('value', 'Next');
	
	var featuresSectionButtons = $('<div></div>');
	featuresSectionButtons.append(reenableExtractorSelectionButton);
	featuresSectionButtons.append(proceedToAlgSelectionButton);
	featuresSectionButtons.hide();
	
	featuresSectionContent.append(featuresSectionButtons);
	
	var targetAlgorithmSectionHeaderText = $('<span></span>').html('Target algorithm selection');
	var targetAlgorithmSectionContent = $('<div></div>');
	var targetAlgorithmSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : targetAlgorithmSectionHeaderText,
		'contentContainer' : targetAlgorithmSectionContent,
		'menuContainer' : targetAlgorithmSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(targetAlgorithmSectionContainer);
	
	newPortfolioBasedSelectorForm.append(targetAlgorithmSectionContainer);
	
	var targetAlgorithmSpinner = $('<div></div>');
	targetAlgorithmSpinner.addClass('spinner');
	targetAlgorithmSpinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
	targetAlgorithmSpinner.hide();
	
	var targetAlgorithmContainer = $('<div></div>');
	targetAlgorithmContainer.hide();
	
	targetAlgorithmSectionContent.append(targetAlgorithmSpinner);
	targetAlgorithmSectionContent.append(targetAlgorithmContainer);
	
	proceedToAlgSelectionButton.click(function() {
		disableMenu(featuresSectionContainer);
		collapseMenu(featuresSectionContainer);
		
		enableMenu(targetAlgorithmSectionContainer);
		expandMenu(targetAlgorithmSectionContainer);
		
		targetAlgorithmContainer.hide();
		targetAlgorithmSpinner.show();
		
		createMultipleAlgorithmSelector({
			'target' : targetAlgorithmContainer,
			'idPrefix' : 'newPortfolioBasedSelector_',
			'description' : 'Please select at least two target algorithms.',
			'instanceSet' : instanceDistributionField.val(),
			'showNewConfigurationButton' : true,
			'forceParameterless' : true,
			'onFinished' : function(response) {
				targetAlgorithmContainer.show();
				targetAlgorithmSpinner.hide();
			},
			'onNext' : function() {
				disableMenu(targetAlgorithmSectionContainer);
				collapseMenu(targetAlgorithmSectionContainer);
				
				enableMenu(designProcedureParametersSectionContainer);
				expandMenu(designProcedureParametersSectionContainer);
				
				createMetricSelector({
					'target' : metricField,
					'onFinished' : function(response) {
						if (response.successful) {
							metricField.change();
						}
					}
				});
				
				parameterValuesContainer.hide();
				designProcedureParametersSpinner.show();
				
				$.post('/hal/getFormSnippet/utilities/getAlgorithmParameterSpaces',
					{
						'hal_httpResponseContentType' : 'json',
						'algorithm' : designProcedureField.val()
					},
					function(response) {
						if (response.successful) {
							scenarioSpace = response.result.parameterSpaces.scenarioSpace;
							configurationSpace = response.result.parameterSpaces.configurationSpace;
							supportedInstanceOptionSpace = response.result.parameterSpaces.supportedInstanceOptionSpace;
							
							createParameterSpaceInput({
								'target' : scenarioParametersSectionContent,
								'idPrefix' : 'newPortfolioBasedSelector_inputParameters_',
								'parameterSpace' : scenarioSpace,
								'fillWithDefaults' : true
							});
							
							createParameterSpaceInput({
								'target' : configurationParametersSectionContent,
								'idPrefix' : 'newPortfolioBasedSelector_inputParameters_',
								'parameterSpace' : configurationSpace,
								'fillWithDefaults' : true
							});
							
							createParameterSpaceInput({
								'target' : supportedInstanceOptionParametersSectionContent,
								'idPrefix' : 'newPortfolioBasedSelector_inputParameters_',
								'parameterSpace' : supportedInstanceOptionSpace,
								'fillWithDefaults' : true
							});

							designProcedureParametersSpinner.hide();
							parameterValuesContainer.show();
						}
					},
					'json'
				);
			},
		});
	});
	
	var designProcedureParametersSectionHeaderText = $('<span></span>').html('Design procedure parameters');
	var designProcedureParametersSectionContent = $('<div></div>');
	var designProcedureParametersSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : designProcedureParametersSectionHeaderText,
		'contentContainer' : designProcedureParametersSectionContent,
		'menuContainer' : designProcedureParametersSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(designProcedureParametersSectionContainer);
	
	newPortfolioBasedSelectorForm.append(designProcedureParametersSectionContainer);
	
	var metricLabel = $('<label></label>');
	metricLabel.attr('for', 'newPortfolioBasedSelector_metric');
	metricLabel.html('Metric');
	
	var metricField = $('<select></select>');
	metricField.attr('name', 'newPortfolioBasedSelector_metric');
	metricField.attr('id', 'newPortfolioBasedSelector_metric');
	metricField.append($('<option></option>').attr('value', '').html('Populating...'));
	
	var metricContainer = $('<div></div>');
	metricContainer.addClass('formElement');
	metricContainer.append(metricLabel);
	metricContainer.append(metricField);
	
	designProcedureParametersSectionContent.append(metricContainer);
	
	var configurationParametersSectionHeaderText = $('<span></span>').html('Configuration settings');
	var configurationParametersSectionContent = $('<div></div>');
	var configurationParametersSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : configurationParametersSectionHeaderText,
		'contentContainer' : configurationParametersSectionContent,
		'menuContainer' : configurationParametersSectionContainer,
		'initialState' : 'collapsed'
	});
	
	var scenarioParametersSectionHeaderText = $('<span></span>').html('Scenario settings');
	var scenarioParametersSectionContent = $('<div></div>');
	var scenarioParametersSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : scenarioParametersSectionHeaderText,
		'contentContainer' : scenarioParametersSectionContent,
		'menuContainer' : scenarioParametersSectionContainer,
		'initialState' : 'collapsed'
	});
	
	var supportedInstanceOptionParametersSectionHeaderText = $('<span></span>').html('Other settings');
	var supportedInstanceOptionParametersSectionContent = $('<div></div>');
	var supportedInstanceOptionParametersSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : supportedInstanceOptionParametersSectionHeaderText,
		'contentContainer' : supportedInstanceOptionParametersSectionContent,
		'menuContainer' : supportedInstanceOptionParametersSectionContainer,
		'initialState' : 'collapsed'
	});
	
	var designProcedureParametersSpinner = $('<div></div>');
	designProcedureParametersSpinner.addClass('spinner');
	designProcedureParametersSpinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
	designProcedureParametersSpinner.hide();
	
	var parameterValuesContainer = $('<div></div>');
	parameterValuesContainer.append(configurationParametersSectionContainer);
	parameterValuesContainer.append(scenarioParametersSectionContainer);
	parameterValuesContainer.append(supportedInstanceOptionParametersSectionContainer);
	parameterValuesContainer.hide();
	
	designProcedureParametersSectionContent.append(designProcedureParametersSpinner);
	designProcedureParametersSectionContent.append(parameterValuesContainer);
	
	var backToTargetAlgorithmsButton = $('<input></input>');
	backToTargetAlgorithmsButton.attr('type', 'button');
	backToTargetAlgorithmsButton.attr('value', 'Back');
	backToTargetAlgorithmsButton.click(function() {
		disableMenu(designProcedureParametersSectionContainer);
		collapseMenu(designProcedureParametersSectionContainer);
		
		enableMenu(targetAlgorithmSectionContainer);
		expandMenu(targetAlgorithmSectionContainer);
	});
	
	var proceedToEnvironmentButton = $('<input></input>');
	proceedToEnvironmentButton.attr('type', 'button');
	proceedToEnvironmentButton.attr('value', 'Next');
	proceedToEnvironmentButton.click(function() {
		disableMenu(designProcedureParametersSectionContainer);
		collapseMenu(designProcedureParametersSectionContainer);
		
		createEnvironmentSelector({
			'target' : environmentField,
			'onFinished' : function(response) {
				if (response.successful) {
					environmentField.change();
				}
			}
		});
		
		enableMenu(environmentSectionContainer);
		expandMenu(environmentSectionContainer);
		buttonsContainer.show();
	});
	
	designProcedureButtons = $('<div></div>');
	designProcedureButtons.append('<hr>');
	designProcedureButtons.append(backToTargetAlgorithmsButton);
	designProcedureButtons.append(proceedToEnvironmentButton);

	designProcedureParametersSectionContent.append(designProcedureButtons);
	
	var environmentSectionHeaderText = $('<span></span>').html('Execution environment');
	var environmentSectionContent = $('<div></div>');
	var environmentSectionContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : environmentSectionHeaderText,
		'contentContainer' : environmentSectionContent,
		'menuContainer' : environmentSectionContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(environmentSectionContainer);
	
	newPortfolioBasedSelectorForm.append(environmentSectionContainer);
	
	var environmentLabel = $('<label></label>');
	environmentLabel.attr('for', 'newPortfolioBasedSelector_environment');
	environmentLabel.html('Environment');
	
	var environmentField = $('<select></select>');
	environmentField.attr('name', 'newPortfolioBasedSelector_environment');
	environmentField.attr('id', 'newPortfolioBasedSelector_environment');
	environmentField.append($('<option></option>').attr('value', '').html('Populating...'));
	
	var environmentContainer = $('<div></div>');
	environmentContainer.addClass('formElement');
	environmentContainer.append(environmentLabel);
	environmentContainer.append(environmentField);
	
	environmentSectionContent.append(environmentContainer);
	
	var statusMessages = $('<div></div>');
	statusMessages.hide();
	
	newPortfolioBasedSelectorForm.append(statusMessages);
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.click(function() {
		statusMessages.hide();
		submitButton.attr('disabled', true);
		
		var formData = newPortfolioBasedSelectorForm.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		
		$.post('/hal/processFormSnippet/createPortfolioBasedSelector/createPortfolioBasedSelector',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					statusMessages.removeClass('alert').removeClass('error').addClass('success');
					statusMessages.html('Successfully created portfolio based selector builder!');
					statusMessages.show();
				} else {
					statusMessages.removeClass('success').removeClass('alert').addClass('error');
					statusMessages.html(response.errorMessage);
					statusMessages.show();
				}
			},
			'json'
		);
	});
	
	var cancelButton = $('<input></input>');
	cancelButton.attr('type', 'button');
	cancelButton.attr('value', 'Cancel');
	cancelButton.click(cancelForm);
	
	var buttonsContainer = $('<div></div>');
	buttonsContainer.addClass('center');
	buttonsContainer.append('<hr>');
	buttonsContainer.append(submitButton);
	buttonsContainer.append(cancelButton);
	buttonsContainer.hide();
	
	newPortfolioBasedSelectorForm.append(buttonsContainer);
	
	var disableAndCollapseAllMenus = function() {
		disableMenu(extractedFeaturesSectionContainer);
		collapseMenu(extractedFeaturesSectionContainer);
		
		disableMenu(featureExtractorSectionContainer);
		collapseMenu(featureExtractorSectionContainer);
		
		disableMenu(featuresSectionContainer);
		collapseMenu(featuresSectionContainer);
	}
	
	var expandAndPopulateFeatureSelectionMenus = function() {
		if (designProcedureField.val() !== "" && designProcedureField.val() !== undefined && designProcedureField.val() !== null) {
			if (instanceDistributionField.val() !== "" && instanceDistributionField.val() !== undefined && instanceDistributionField.val() !== null) {
				// enable and populate the feature sub-menus.
				enableMenu(featuresSectionContainer);
				expandMenu(featuresSectionContainer);
			
				enableMenu(featureExtractorSectionContainer);
				expandMenu(featureExtractorSectionContainer);
				
				featureExtractorSelectorContainer.hide();
				featureExtractorSpinner.show();
				
				createMultipleFeatureExtractorSelector({
					'target' : featureExtractorSelectorContainer,
					'idPrefix' : 'newPortfolioBasedSelector_',
					'description' : 'Please select at least one feature extractor, and choose the features for the selector.',
					'instanceSet' : instanceDistributionField.val(),
					'onFinished' : function(response) {
						featureExtractorSelectorContainer.show();
						featureExtractorSpinner.hide();
					},
					'onNext' : function(extractedFeatures) {
						// count up the features used in all of the selected features.
						extractedFeaturesSectionHeaderText.html(numberOfProperties(extractedFeatures)+' extracted features');
						extractedFeaturesSectionContent.html('');
						for (var i in extractedFeatures) {
							var label = $('<label></label>');
							label.attr('for', 'newPortfolioBasedSelector_useFeature_'+i);
							label.html(i);
							
							var field = $('<input></input>');
							field.attr('id', 'newPortfolioBasedSelector_useFeature_'+i);
							field.attr('name', 'newPortfolioBasedSelector_useFeature_'+i);
							field.attr('type', 'checkbox');
							field.attr('checked', 'on');
							
							var element = $('<div></div>');
							element.addClass('formElementInline');
							element.append(field);
							element.append(label);

							
							extractedFeaturesSectionContent.append(element);
						}
						
						disableMenu(featureExtractorSectionContainer);
						collapseMenu(featureExtractorSectionContainer);
						
						enableMenu(extractedFeaturesSectionContainer);
						collapseMenu(extractedFeaturesSectionContainer);
						
						featuresSectionButtons.show();
					},
				});
				
				disableMenu(extractedFeaturesSectionContainer);
				collapseMenu(extractedFeaturesSectionContainer);
			} else {
				disableAndCollapseAllMenus();
			}
		} else {
			disableAndCollapseAllMenus();
		}
	}
	
	createInstanceDistributionSelector({
		'target' : instanceDistributionField,
		'onFinished' : function(response) {
			if (response.successful) {
				instanceDistributionField.change(expandAndPopulateFeatureSelectionMenus);
				instanceDistributionField.change();
			}
		}
	});
	
	designProcedureField.change(expandAndPopulateFeatureSelectionMenus);
	
	options.target.html(newPortfolioBasedSelectorForm);
}