function createUploadFileForm(options) {
	var uploadFileForm = $('<form></form>');
	uploadFileForm.attr('id', 'uploadFile');
	uploadFileForm.attr('method', 'post');
	uploadFileForm.attr('enctype', 'multipart/form-data');
	
	var uploadFileFormAllContents = $('<div></div>');
	uploadFileFormAllContents.addClass('formContent');
	
	uploadFileFormAllContents.append(uploadFileForm);
	uploadFileFormAllContents.hide();
	
	var uploadFileContents = $('<div></div>');
	uploadFileContents.attr('id', 'uploadFile_formContent');
	uploadFileForm.append(uploadFileContents);
	
	// File field
	var uploadFileField = $('<div></div>');
	uploadFileField.addClass('formElement');
	
	var uploadFileFieldLabel = $('<label></label>');
	uploadFileFieldLabel.attr('for', 'uploadFile_uploadedFile');
	uploadFileFieldLabel.html('File to upload');
	
	var uploadFileFieldDescription = $('<div></div>');
	uploadFileFieldDescription.addClass('formFieldDescription');
	uploadFileFieldDescription.html('Choose a file to upload.');
	
	var uploadFileFieldInput = $('<input></input>');
	uploadFileFieldInput.attr('id', 'uploadFile_uploadedFile');
	uploadFileFieldInput.attr('name', 'uploadFile_uploadedFile');
	uploadFileFieldInput.attr('type', 'file');
	
	uploadFileField.append(uploadFileFieldLabel);
	uploadFileField.append(uploadFileFieldDescription);
	uploadFileField.append(uploadFileFieldInput);
	
	uploadFileForm.append(uploadFileField);
	//////
	
	// Destination field
	if (options.showDestinationField) {
		var fileDestinationField = $('<div></div>');
		fileDestinationField.addClass('formElement');

		var fileDestinationFieldLabel = $('<label></label>');
		fileDestinationFieldLabel.attr('for', 'uploadFile_fileDestination');
		fileDestinationFieldLabel.html('Destination directory');

		var fileDestinationFieldDescription = $('<div></div>');
		fileDestinationFieldDescription.addClass('formFieldDescription');
		fileDestinationFieldDescription.html('Choose a destination directory. All zip archives will be extracted after the upload and move have completed.');

		var fileDestinationFieldInputContent = $('<div></div>');

		var fileDestinationFieldPrefixSpan = $('<span></span>');
		fileDestinationFieldPrefixSpan.html('HAL_DIRECTORY/uploads/');

		var fileDestinationFieldInput = $('<input></input>');
		fileDestinationFieldInput.attr('name', 'uploadFile_fileDestination');
		fileDestinationFieldInput.attr('type', 'text');
		fileDestinationFieldInput.attr('size', 100);

		fileDestinationFieldInputContent.append(fileDestinationFieldPrefixSpan);
		fileDestinationFieldInputContent.append(fileDestinationFieldInput);

		fileDestinationField.append(fileDestinationFieldLabel);
		fileDestinationField.append(fileDestinationFieldDescription);
		fileDestinationField.append(fileDestinationFieldInputContent);

		uploadFileForm.append(fileDestinationField);
	}
	//////
	
	// Status monitoring
	var uploadFileStatusMessages = $('<div></div>');
	uploadFileStatusMessages.hide();
	
	uploadFileForm.append(uploadFileStatusMessages);
	//////

	var uploadProgressContainer = $('<div></div>');
	uploadProgressContainer.addClass('formElement');
	
	var uploadProgressLabel = $('<label></label>');
	uploadProgressLabel.attr('for', 'uploadFile_fileUploadProgressBar');
	uploadProgressLabel.html('File upload progress');
	
	var uploadProgressDescription = $('<div></div>');
	uploadProgressDescription.addClass('formFieldDescription');

	var progressBarContainer = $('<div></div>');
	
	var uploadProgressBar = $('<div></div>');
	uploadProgressBar.addClass('runProgressBar');
	uploadProgressBar.attr('id', 'uploadFile_fileUploadProgressBar');
	
	var uploadProgressBarText = $('<div></div>');
	uploadProgressBarText.addClass('progressBarText');
	
	progressBarContainer.append(uploadProgressBar);
	progressBarContainer.append(uploadProgressBarText);
	progressBarContainer.append($('<div></div>').addClass('clear'));
	
	var uploadedFilesName = $('<span></span>');
	uploadedFilesName.append("Uploaded files");
	
	var uploadedFilesNameLink = $('<a></a>');
	uploadedFilesNameLink.append(uploadedFilesName);
	
	var uploadedFilesHeader = $('<div></div>');
	uploadedFilesHeader.addClass('collapsed');
	uploadedFilesHeader.append(uploadedFilesNameLink);
	
	var uploadedFilesContent = $('<div></div>');
	uploadedFilesContent.addClass('collapsibleContent');
	
	var uploadedFilesContainer = $('<div></div>');
	uploadedFilesContainer.addClass('collapsible');
	uploadedFilesContainer.attr('style', 'margin-top:10px;');
	
	uploadedFilesContainer.append(uploadedFilesHeader);
	uploadedFilesContainer.append(uploadedFilesContent);
	
	collapsible(uploadedFilesContainer);
	uploadedFilesContainer.hide();
	
	uploadProgressContainer.append(uploadProgressLabel);
	uploadProgressContainer.append(uploadProgressDescription);
	uploadProgressContainer.append(progressBarContainer);
	uploadProgressContainer.append(uploadedFilesContainer);
	uploadProgressContainer.hide();
	
	var firstProgressUpdate = true;
	
	var updateUploadProgress = function() {
		$.post('/hal/commands/uploadFile/',
			{
				'getFileUploadProgress' : true
			},
			function(response) {
				if (response.successful) {
					if (firstProgressUpdate) {
						firstProgressUpdate = false;
						
						uploadFileStatusMessages.html('');
						uploadFileStatusMessages.append(uploadProgressContainer);
						uploadProgressContainer.slideDown();
					}
					
					var description = "Unknown state.";
					if ("RESERVED" === response.status) {
						description = "Reserved upload id.";
					} else if ("UNSTARTED" === response.status) {
						description = "Waiting for upload to start.";
					} else if ("UPLOADING" === response.status) {
						description = "Uploading file(s).";
					} else if ("MOVING" === response.status) {
						description = "Moving uploaded file(s) to the correct location.";
					} else if ("EXTRACTING" === response.status) {
						description = "Extracting archive file(s).";
					} else if ("FINISHED" === response.status) {
						description = "Finished.";
					}
					
					uploadProgressDescription.html(description);
					
					if (response.isProgressDeterminate && response.percentageComplete !== undefined) {
						var percentageComplete = response.percentageComplete;
						var barOptions = {
								steps : 100,
								width : 120,
								showText : false,
								boxImage : '/hal/commands/getFile/web/icons/progressbar.gif',
								barImage : '/hal/commands/getFile/web/icons/progressbg_halBlue.gif'
						};
						
						
						uploadProgressBar.progressBar(parseInt(percentageComplete), barOptions);
						
						uploadProgressBarText.html(response.megabytesRead.toFixed(3)+' / '+response.contentLength.toFixed(3)+' MB');
					}
					
					if (response.uploadedFiles !== undefined) {
						uploadedFilesContent.html('');
						
						var list = $('<ul></ul>');
						for (var i in response.uploadedFiles) {
							list.append('<li>'+response.uploadedFiles[i]+'</li>');
						}
						
						uploadedFilesContent.append(list);
						uploadedFilesContainer.slideDown();
					} else {
						uploadedFilesContainer.slideUp();
					}
				} else {
					
				}
			},
			'json');
	};
	
	var updateUploadInterval;
	
	var beforeFormSubmission = function() {
		var spinnerDiv = $('<div></div>');
		spinnerDiv.addClass('spinner');
		spinnerDiv.addClass('center');
		spinnerDiv.html($('<img></img').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').attr('alt', 'Uploading. Please wait.'));
		
		uploadFileStatusMessages.removeClass('alert').removeClass('error');
		uploadFileStatusMessages.addClass('success');
		uploadFileStatusMessages.html(spinnerDiv);
		uploadFileStatusMessages.slideDown();
		
		updateUploadInterval = setInterval(updateUploadProgress, 1000);
	};
	
	var formResponse = function(response) {
		uploadFileUploadButton.attr('disabled', false);

		setTimeout(function (a,b) {
			  clearInterval(updateUploadInterval);
			},1500);
		
		if (response.successful) {
			uploadFileStatusMessages.removeClass('alert').removeClass('error');
			uploadFileStatusMessages.addClass('success');
			
			if ($('#formParent').children().size() > 1) {
				$('#formParent').children().last().slideUp();
				$('#formParent').children().last().remove();
				$('#formParent').children().last().slideDown();
			} else {
				uploadFileUploadButton.hide();
				uploadAnotherFileButton.show();
			}
			
			if (options.callbackFunction !== undefined) {
				options.callbackFunction(response.uploadedFiles);
			}
		} else {
			uploadFileStatusMessages.removeClass('alert').removeClass('success');
			uploadFileStatusMessages.addClass('error');
			
			uploadFileStatusMessages.html($('<div>').html(response.errorMessage).text());
			uploadFileStatusMessages.show();
		}
	};
		
	// Buttons
	var uploadFileFormButtons = $('<div></div>');
	uploadFileFormButtons.addClass('center');
	
	var uploadAnotherFileButton = $('<input></input>');
	uploadAnotherFileButton.attr('type', 'button');
	uploadAnotherFileButton.attr('value', 'Upload another file');
	uploadAnotherFileButton.click(function() {
		window.location.href = '/hal/uploadFile';
	});
	uploadAnotherFileButton.hide();
	
	var uploadFileUploadButton = $('<input></input>');
	uploadFileUploadButton.attr('type', 'button');
	uploadFileUploadButton.attr('value', 'Upload');
	uploadFileUploadButton.click(function() {
		uploadFileUploadButton.attr('disabled', true);
		
		$.post('/hal/commands/uploadFile/',
			{
				'requestUploadID' : true
			},
			function(response) {
				if (response.successful) {
					var formOptions = {
						'beforeSubmit' : beforeFormSubmission,
						'success' : formResponse,
						'dataType' : 'json',
						'type' : 'POST',
						'url' : '/hal/commands/uploadFile/',
						'data' : {
							'uploadFile_extractArchiveContents' : options.extractArchiveContents
						},
						'clearForm' : false,
						'resetForm' : false
					};
					
					uploadFileForm.ajaxSubmit(formOptions);
					
					uploadFileField.slideUp();
					if (options.showDestinationField) {
						fileDestinationField.slideUp();
					}
					uploadProgressContainer.slideDown();
				}
			},
			'json');
	});
	
	var uploadFileCancelButton = $('<input></input>');
	uploadFileCancelButton.attr('type', 'button');
	uploadFileCancelButton.attr('value', 'Cancel');
	uploadFileCancelButton.click(cancelForm);
	
	uploadFileFormButtons.append($('<hr>'));
	uploadFileFormButtons.append(uploadAnotherFileButton);
	uploadFileFormButtons.append(uploadFileUploadButton);
	uploadFileFormButtons.append(uploadFileCancelButton);
	
	uploadFileForm.append(uploadFileFormButtons);
	//////
	
	options.target.append(uploadFileFormAllContents);
	uploadFileFormAllContents.show();
}